@@ -1,5 +1,5 @@
 use std::{convert::TryInto, str::FromStr};
-use test_env_log::test;
+use test_log::test;
 
 use cao_lang::{
     compiler::{CallNode, IntegerNode, StringNode, VarNode},
@@ -63,7 +63,7 @@ fn test_native_w_table_input() {
     }
 
     let myboi = move |vm: &mut Vm<State>, table: &FieldTable| {
-        let key = Key::from_str("boi").unwrap();
+        let key = Handle::from_str("boi").unwrap();
         let res = table.get_value(key).unwrap_or_default();
         if let Value::Integer(i) = res {
             vm.get_aux_mut().param = i;
